@@ -15,7 +15,7 @@ from sca2d.scadclasses import (UseIncStatment,
                                ModuleDef,
                                FunctionDef)
 from sca2d.messages import Message
-from sca2d.definitions import SCAD_VARS, SCAD_MODS, SCAD_FUNCS
+from sca2d.definitions import SCAD_VARS, SCAD_MODS, SCAD_FUNCS, CustomArgDef
 from sca2d.patterns import GOBAL_PATTERN
 
 class OuterScope(ScopeContents):
@@ -404,9 +404,8 @@ class BuiltInScope(OuterScope):
     def _set_built_in_mods(self):
         for scad_mod in SCAD_MODS:
             tree = DummyTree()
-            module = ModuleDef(name=scad_mod,
-                               n_args=0,
-                               n_kwargs=0,
+            module = ModuleDef(name=scad_mod[0],
+                               arg_defintion=UndefinedArgDef(),
                                tree=tree,
                                scope=self)
             self._defined_modules.append(module)
@@ -414,11 +413,19 @@ class BuiltInScope(OuterScope):
     def _set_built_in_funcs(self):
         for scad_func in SCAD_FUNCS:
             tree = DummyTree()
-            function = FunctionDef(name=scad_func,
-                                   n_args=0,
-                                   n_kwargs=0,
+            function = FunctionDef(name=scad_func[0],
+                                   arg_defintion=UndefinedArgDef(),
                                    tree=tree,
                                    scope=self)
             self._defined_functions.append(function)
 
+class UndefinedArgDef(CustomArgDef):
+    '''
+    Custom argument defitionion for use when behaviour is undefined
+    so no warnings can be raised. Not checks are performed
+    '''
+
+    def check_call(self, call):
+        return []
+
 BUILT_IN_SCOPE = BuiltInScope()
