@@ -5,8 +5,7 @@ information about certain objects parsed from the .scad file.
 
 from dataclasses import dataclass
 from lark import Tree, Token
-
-from sca2d import utilities
+from sca2d.definitions import CustomArgDef
 
 class DummyTree():
     '''
@@ -106,60 +105,86 @@ class DummyToken():
         '''
         return self._column
 
-#TODO: make a base class for all of these and make the __eq__ method safer!
+class ScadDef:
+    '''
+    A base class for ModuleDef and FunctionDef
+    '''
 
-@dataclass
-class ModuleDef:
+    def __init__(self, name, arg_defintion, tree, scope, included_by=None):
+        self.name = name
+        self._arg_defintion = arg_defintion
+        self.tree = tree
+        self.scope = scope
+        self.included_by = included_by
+
+    def check_call(self, call):
+        '''
+        For a given module call check if input arguments are defined appropriatly.
+        Return a list of messages for any issue.
+        '''
+        if isinstance(self._arg_defintion, CustomArgDef):
+            return self._arg_defintion.check_call(call)
+
+        messages = []
+
+        n_args_called = len([arg for arg in call.assigned_arguments if not arg.is_special()])
+        n_args = len(self._arg_defintion)
+        n_kwargs = len([arg for arg in self._arg_defintion if arg.is_kwarg])
+        if n_args_called > n_args:
+            messages.append(['W3001', call.tree])
+        elif n_args_called < n_args-n_kwargs:
+            messages.append(['W3002', call.tree])
+        else:
+            assgined = [0]*n_args
+            for i, arg in enumerate(call.assigned_arguments):
+                if isinstance(arg, UnnamedArgument):
+                    assgined[i] += 1
+                else:
+                    if arg in self._arg_defintion:
+                        assgined[self._arg_defintion.index(arg)] += 1
+                    elif not arg.is_special():
+                        messages.append(['W3003', arg.tree, [arg.name]])
+            for arg, times_assigned in zip(self._arg_defintion, assgined):
+                if times_assigned == 0:
+                    if not arg.is_kwarg:
+                        messages.append(['W3004', call.tree, [arg.name]])
+                if times_assigned > 1:
+                    messages.append(['W3005', call.tree, [arg.name]])
+
+
+        return messages
+
+    def __str__(self):
+        return self.name
+
+
+    def __eq__(self, other):
+        if isinstance(other, str):
+            return self.name == other
+        return self.name == other.name
+
+
+class ModuleDef(ScadDef):
     '''
     A class for a module definition. Contains the name of the defined module.
     The number of args (inc. kwargs) and the number of kwargs. The original
     Lark tree and and the ScopeContents for this definition.
     '''
-    name: str
-    # total number of arguments including keyword arguments
-    n_args: int
-    n_kwargs: int
-    tree: Tree
-    scope: object
-    included_by: Tree = None
-
-    def __str__(self):
-        return self.name
 
     def __repr__(self):
         return "<sca2d.scadclasses.ModuleDef "+self.name+">"
 
-    def __eq__(self, other):
-        if isinstance(other, str):
-            return self.name == other
-        return self.name == other.name
 
-
-@dataclass
-class FunctionDef:
+class FunctionDef(ScadDef):
     '''
     A class for a function definition. Contains the name of the defined function
     The number of args (inc. kwargs) and the number of kwargs. The original
     Lark tree and and the ScopeContents for this definition.
     '''
-    name: str
-    # total number of arguments including keyword arguments
-    n_args: int
-    n_kwargs: int
-    tree: Tree
-    scope: object
-    included_by: Tree = None
-
-    def __str__(self):
-        return self.name
 
     def __repr__(self):
         return "<sca2d.scadclasses.FunctionDef "+self.name+">"
 
-    def __eq__(self, other):
-        if isinstance(other, str):
-            return self.name == other
-        return self.name == other.name
 
 @dataclass
 class ModuleCall:
@@ -172,8 +197,7 @@ class ModuleCall:
     '''
     name: str
     # total number of arguments including keyword arguments
-    n_args: int
-    n_kwargs: int
+    assigned_arguments: list
     tree: Tree
     scope: object
 
@@ -215,8 +239,7 @@ class FunctionCall:
     '''
     name: str
     # total number of arguments including keyword arguments
-    n_args: int
-    n_kwargs: int
+    assigned_arguments: list
     tree: Tree
 
     def __str__(self):
@@ -248,6 +271,8 @@ class Variable:
         self.name = token.value
         self.token = token
         self.included_by = None
+        self.is_kwarg = False
+        self.default_is_undef = False
 
     def __str__(self):
         return self.name
@@ -296,3 +321,18 @@ class UseIncStatment:
         if isinstance(other, UseIncStatment):
             return self.filename == other.filename
         return False
+
+
+class UnnamedArgument:
+    '''
+    A class for input arguments that are unamed
+    '''
+    def __init__(self, tree):
+        self.tree = tree
+
+    def is_special(self):
+        """
+        Returns whether this is an OpenSCAD "special variable" , an unamed argument is
+        never a special variable
+        """
+        return False
